import isBuiltinModule from 'is-builtin-module';
import { createLogger } from '@niceties/logger';

function getOptionsObject(options, defaults, factory) {
    const result = Object.assign(Object.assign({}, defaults), options);
    if (factory) {
        for (const [k, v] of Object.entries(factory)) {
            result[k] = v(result, k);
        }
    }
    return result;
}

function logger (options) {
    return createLogger(options.pluginName);
}

const factories = { logger };
function index (options = {}) {
    const { pluginName, external, logger, verbose } = getOptionsObject(options, {
        pluginName: '@rollup-extras/plugin-externals',
        verbose: false
    }, factories);
    const logLevel = verbose ? 1 /* LogLevel.info */ : 0 /* LogLevel.verbose */;
    return {
        name: pluginName,
        resolveId(id) {
            let isExternal = id.includes('node_modules') || isBuiltinModule(id);
            if (external) {
                isExternal = external(id, isExternal);
            }
            logger(`'${id}' is ${isExternal ? '' : 'not '}external`, logLevel);
            return isExternal ? false : null;
        }
    };
}

export { index as default };
